@@ -10,8 +10,7 @@
 
 // compile-flags: -C debug_assertions=yes
 // ignore-wasm32-bare compiled with panic=abort by default
-
-#![feature(i128_type)]
+// ignore-emscripten dies with an LLVM error
 
 use std::panic;
 
