@@ -5,16 +5,16 @@
 
 use rustc::lint::*;
 use rustc::hir;
-use rustc::hir::{Expr, Expr_, QPath};
+use rustc::hir::{Expr, Expr_, QPath, Ty_, Pat, PatKind, BindingAnnotation, StmtSemi, StmtExpr, StmtDecl, Decl_, Stmt};
 use rustc::hir::intravisit::{NestedVisitorMap, Visitor};
-use syntax::ast::{self, Attribute, LitKind, NodeId, DUMMY_NODE_ID};
-use syntax::codemap::Span;
+use syntax::ast::{self, Attribute, LitKind, DUMMY_NODE_ID};
 use std::collections::HashMap;
 
 /// **What it does:** Generates clippy code that detects the offending pattern
 ///
 /// **Example:**
 /// ```rust
+/// // ./tests/ui/my_lint.rs
 /// fn foo() {
 ///     // detect the following pattern
 ///     #[clippy(author)]
@@ -25,9 +25,11 @@ use std::collections::HashMap;
 /// }
 /// ```
 ///
-/// prints
+/// Running `TESTNAME=ui/my_lint cargo test --test compile-test` will produce
+/// a `./tests/ui/new_lint.stdout` file with the generated code:
 ///
-/// ```
+/// ```rust
+/// // ./tests/ui/new_lint.stdout
 /// if_chain!{
 ///     if let Expr_::ExprIf(ref cond, ref then, None) = item.node,
 ///     if let Expr_::ExprBinary(BinOp::Eq, ref left, ref right) = cond.node,
@@ -39,9 +41,9 @@ use std::collections::HashMap;
 ///     }
 /// }
 /// ```
-declare_lint! {
+declare_clippy_lint! {
     pub LINT_AUTHOR,
-    Warn,
+    internal_warn,
     "helper for writing lints"
 }
 
@@ -171,6 +173,12 @@ impl PrintVisitor {
             },
         }
     }
+
+    fn print_qpath(&mut self, path: &QPath) {
+        print!("    if match_qpath({}, &[", self.current);
+        print_path(path, &mut true);
+        println!("]);");
+    }
 }
 
 struct PrintVisitor {
@@ -260,9 +268,17 @@ impl<'tcx> Visitor<'tcx> for PrintVisitor {
                     },
                 }
             },
-            Expr_::ExprCast(ref expr, ref _ty) => {
+            Expr_::ExprCast(ref expr, ref ty) => {
                 let cast_pat = self.next("expr");
-                println!("Cast(ref {}, _) = {};", cast_pat, current);
+                let cast_ty = self.next("cast_ty");
+                let qp_label = self.next("qp");
+
+                println!("Cast(ref {}, ref {}) = {};", cast_pat, cast_ty, current);
+                if let Ty_::TyPath(ref qp) = ty.node {
+                    println!("    if let Ty_::TyPath(ref {}) = {}.node;", qp_label, cast_ty);
+                    self.current = qp_label;
+                    self.print_qpath(qp);
+                }
                 self.current = cast_pat;
                 self.visit_expr(expr);
             },
@@ -288,17 +304,47 @@ impl<'tcx> Visitor<'tcx> for PrintVisitor {
                 self.current = then_pat;
                 self.visit_expr(then);
             },
-            Expr_::ExprWhile(ref _cond, ref _body, ref _opt_label) => {
-                println!("While(ref cond, ref body, ref opt_label) = {};", current);
-                println!("    // unimplemented: `ExprWhile` is not further destructured at the moment");
+            Expr_::ExprWhile(ref cond, ref body, _) => {
+                let cond_pat = self.next("cond");
+                let body_pat = self.next("body");
+                let label_pat = self.next("label");
+                println!("While(ref {}, ref {}, ref {}) = {};", cond_pat, body_pat, label_pat, current);
+                self.current = cond_pat;
+                self.visit_expr(cond);
+                self.current = body_pat;
+                self.visit_block(body);
             },
-            Expr_::ExprLoop(ref _body, ref _opt_label, ref _desuraging) => {
-                println!("Loop(ref body, ref opt_label, ref desugaring) = {};", current);
-                println!("    // unimplemented: `ExprLoop` is not further destructured at the moment");
+            Expr_::ExprLoop(ref body, _, desugaring) => {
+                let body_pat = self.next("body");
+                let des = loop_desugaring_name(desugaring);
+                let label_pat = self.next("label");
+                println!("Loop(ref {}, ref {}, {}) = {};", body_pat, label_pat, des, current);
+                self.current = body_pat;
+                self.visit_block(body);
             },
-            Expr_::ExprMatch(ref _expr, ref _arms, ref _desugaring) => {
-                println!("Match(ref expr, ref arms, ref desugaring) = {};", current);
-                println!("    // unimplemented: `ExprMatch` is not further destructured at the moment");
+            Expr_::ExprMatch(ref expr, ref arms, desugaring) => {
+                let des = desugaring_name(desugaring);
+                let expr_pat = self.next("expr");
+                let arms_pat = self.next("arms");
+                println!("Match(ref {}, ref {}, {}) = {};", expr_pat, arms_pat, des, current);
+                self.current = expr_pat;
+                self.visit_expr(expr);
+                println!("    if {}.len() == {};", arms_pat, arms.len());
+                for (i, arm) in arms.iter().enumerate() {
+                    self.current = format!("{}[{}].body", arms_pat, i);
+                    self.visit_expr(&arm.body);
+                    if let Some(ref guard) = arm.guard {
+                        let guard_pat = self.next("guard");
+                        println!("    if let Some(ref {}) = {}[{}].guard", guard_pat, arms_pat, i);
+                        self.current = guard_pat;
+                        self.visit_expr(guard);
+                    }
+                    println!("    if {}[{}].pats.len() == {};", arms_pat, i, arm.pats.len());
+                    for (j, pat) in arm.pats.iter().enumerate() {
+                        self.current = format!("{}[{}].pats[{}]", arms_pat, i, j);
+                        self.visit_pat(pat);
+                    }
+                }
             },
             Expr_::ExprClosure(ref _capture_clause, ref _func, _, _, _) => {
                 println!("Closure(ref capture_clause, ref func, _, _, _) = {};", current);
@@ -310,7 +356,7 @@ impl<'tcx> Visitor<'tcx> for PrintVisitor {
                 self.current = sub_pat;
                 self.visit_expr(sub);
             },
-            Expr_::ExprBlock(ref block) => {
+            Expr_::ExprBlock(ref block, _) => {
                 let block_pat = self.next("block");
                 println!("Block(ref {}) = {};", block_pat, current);
                 self.current = block_pat;
@@ -344,14 +390,6 @@ impl<'tcx> Visitor<'tcx> for PrintVisitor {
                 self.current = obj_pat;
                 self.visit_expr(object);
             },
-            Expr_::ExprTupField(ref object, ref field_id) => {
-                let obj_pat = self.next("object");
-                let field_id_pat = self.next("field_id");
-                println!("TupField(ref {}, ref {}) = {};", obj_pat, field_id_pat, current);
-                println!("    if {}.node == {}", field_id_pat, field_id.node);
-                self.current = obj_pat;
-                self.visit_expr(object);
-            },
             Expr_::ExprIndex(ref object, ref index) => {
                 let object_pat = self.next("object");
                 let index_pat = self.next("index");
@@ -365,7 +403,7 @@ impl<'tcx> Visitor<'tcx> for PrintVisitor {
                 let path_pat = self.next("path");
                 println!("Path(ref {}) = {};", path_pat, current);
                 self.current = path_pat;
-                self.visit_qpath(path, expr.id, expr.span);
+                self.print_qpath(path);
             },
             Expr_::ExprAddrOf(mutability, ref inner) => {
                 let inner_pat = self.next("inner");
@@ -420,7 +458,7 @@ impl<'tcx> Visitor<'tcx> for PrintVisitor {
                     println!("Struct(ref {}, ref {}, None) = {};", path_pat, fields_pat, current);
                 }
                 self.current = path_pat;
-                self.visit_qpath(path, expr.id, expr.span);
+                self.print_qpath(path);
                 println!("    if {}.len() == {};", fields_pat, fields.len());
                 println!("    // unimplemented: field checks");
             },
@@ -435,11 +473,160 @@ impl<'tcx> Visitor<'tcx> for PrintVisitor {
         }
     }
 
-    fn visit_qpath(&mut self, path: &QPath, _: NodeId, _: Span) {
-        print!("    if match_qpath({}, &[", self.current);
-        print_path(path, &mut true);
-        println!("]);");
+    fn visit_pat(&mut self, pat: &Pat) {
+        print!("    if let PatKind::");
+        let current = format!("{}.node", self.current);
+        match pat.node {
+            PatKind::Wild => println!("Wild = {};", current),
+            PatKind::Binding(anno, _, name, ref sub) => {
+                let anno_pat = match anno {
+                    BindingAnnotation::Unannotated => "BindingAnnotation::Unannotated",
+                    BindingAnnotation::Mutable => "BindingAnnotation::Mutable",
+                    BindingAnnotation::Ref => "BindingAnnotation::Ref",
+                    BindingAnnotation::RefMut => "BindingAnnotation::RefMut",
+                };
+                let name_pat = self.next("name");
+                if let Some(ref sub) = *sub {
+                    let sub_pat = self.next("sub");
+                    println!("Binding({}, _, {}, Some(ref {})) = {};", anno_pat, name_pat, sub_pat, current);
+                    self.current = sub_pat;
+                    self.visit_pat(sub);
+                } else {
+                    println!("Binding({}, _, {}, None) = {};", anno_pat, name_pat, current);
+                }
+                println!("    if {}.node.as_str() == \"{}\";", name_pat, name.node.as_str());
+            }
+            PatKind::Struct(ref path, ref fields, ignore) => {
+                let path_pat = self.next("path");
+                let fields_pat = self.next("fields");
+                println!("Struct(ref {}, ref {}, {}) = {};", path_pat, fields_pat, ignore, current);
+                self.current = path_pat;
+                self.print_qpath(path);
+                println!("    if {}.len() == {};", fields_pat, fields.len());
+                println!("    // unimplemented: field checks");
+            }
+            PatKind::TupleStruct(ref path, ref fields, skip_pos) => {
+                let path_pat = self.next("path");
+                let fields_pat = self.next("fields");
+                println!("TupleStruct(ref {}, ref {}, {:?}) = {};", path_pat, fields_pat, skip_pos, current);
+                self.current = path_pat;
+                self.print_qpath(path);
+                println!("    if {}.len() == {};", fields_pat, fields.len());
+                println!("    // unimplemented: field checks");
+            },
+            PatKind::Path(ref path) => {
+                let path_pat = self.next("path");
+                println!("Path(ref {}) = {};", path_pat, current);
+                self.current = path_pat;
+                self.print_qpath(path);
+            }
+            PatKind::Tuple(ref fields, skip_pos) => {
+                let fields_pat = self.next("fields");
+                println!("Tuple(ref {}, {:?}) = {};", fields_pat, skip_pos, current);
+                println!("    if {}.len() == {};", fields_pat, fields.len());
+                println!("    // unimplemented: field checks");
+            }
+            PatKind::Box(ref pat) => {
+                let pat_pat = self.next("pat");
+                println!("Box(ref {}) = {};", pat_pat, current);
+                self.current = pat_pat;
+                self.visit_pat(pat);
+            },
+            PatKind::Ref(ref pat, muta) => {
+                let pat_pat = self.next("pat");
+                println!("Ref(ref {}, Mutability::{:?}) = {};", pat_pat, muta, current);
+                self.current = pat_pat;
+                self.visit_pat(pat);
+            },
+            PatKind::Lit(ref lit_expr) => {
+                let lit_expr_pat = self.next("lit_expr");
+                println!("Lit(ref {}) = {}", lit_expr_pat, current);
+                self.current = lit_expr_pat;
+                self.visit_expr(lit_expr);
+            }
+            PatKind::Range(ref start, ref end, end_kind) => {
+                let start_pat = self.next("start");
+                let end_pat = self.next("end");
+                println!("Range(ref {}, ref {}, RangeEnd::{:?}) = {};", start_pat, end_pat, end_kind, current);
+                self.current = start_pat;
+                self.visit_expr(start);
+                self.current = end_pat;
+                self.visit_expr(end);
+            }
+            PatKind::Slice(ref start, ref middle, ref end) => {
+                let start_pat = self.next("start");
+                let end_pat = self.next("end");
+                if let Some(ref middle) = middle {
+                    let middle_pat = self.next("middle");
+                    println!("Slice(ref {}, Some(ref {}), ref {}) = {};", start_pat, middle_pat, end_pat, current);
+                    self.current = middle_pat;
+                    self.visit_pat(middle);
+                } else {
+                    println!("Slice(ref {}, None, ref {}) = {};", start_pat, end_pat, current);
+                }
+                println!("    if {}.len() == {};", start_pat, start.len());
+                for (i, pat) in start.iter().enumerate() {
+                    self.current = format!("{}[{}]", start_pat, i);
+                    self.visit_pat(pat);
+                }
+                println!("    if {}.len() == {};", end_pat, end.len());
+                for (i, pat) in end.iter().enumerate() {
+                    self.current = format!("{}[{}]", end_pat, i);
+                    self.visit_pat(pat);
+                }
+            }
+        }
     }
+
+    fn visit_stmt(&mut self, s: &Stmt) {
+        print!("    if let Stmt_::");
+        let current = format!("{}.node", self.current);
+        match s.node {
+            // Could be an item or a local (let) binding:
+            StmtDecl(ref decl, _) => {
+                let decl_pat = self.next("decl");
+                println!("StmtDecl(ref {}, _) = {}", decl_pat, current);
+                print!("    if let Decl_::");
+                let current = format!("{}.node", decl_pat);
+                match decl.node {
+                    // A local (let) binding:
+                    Decl_::DeclLocal(ref local) => {
+                        let local_pat = self.next("local");
+                        println!("DeclLocal(ref {}) = {};", local_pat, current);
+                        if let Some(ref init) = local.init {
+                            let init_pat = self.next("init");
+                            println!("    if let Some(ref {}) = {}.init", init_pat, local_pat);
+                            self.current = init_pat;
+                            self.visit_expr(init);
+                        }
+                        self.current = format!("{}.pat", local_pat);
+                        self.visit_pat(&local.pat);
+                    },
+                    // An item binding:
+                    Decl_::DeclItem(_) => {
+                        println!("DeclItem(item_id) = {};", current);
+                    },
+                }
+            }
+
+            // Expr without trailing semi-colon (must have unit type):
+            StmtExpr(ref e, _) => {
+                let e_pat = self.next("e");
+                println!("StmtExpr(ref {}, _) = {}", e_pat, current);
+                self.current = e_pat;
+                self.visit_expr(e);
+            },
+
+            // Expr with trailing semi-colon (may have any type):
+            StmtSemi(ref e, _) => {
+                let e_pat = self.next("e");
+                println!("StmtSemi(ref {}, _) = {}", e_pat, current);
+                self.current = e_pat;
+                self.visit_expr(e);
+            },
+        }
+    }
+
     fn nested_visit_map<'this>(&'this mut self) -> NestedVisitorMap<'this, 'tcx> {
         NestedVisitorMap::None
     }
@@ -449,13 +636,31 @@ fn has_attr(attrs: &[Attribute]) -> bool {
     attrs.iter().any(|attr| {
         attr.check_name("clippy") && attr.meta_item_list().map_or(false, |list| {
             list.len() == 1 && match list[0].node {
-                ast::NestedMetaItemKind::MetaItem(ref it) => it.name == "author",
+                ast::NestedMetaItemKind::MetaItem(ref it) => it.name() == "author",
                 ast::NestedMetaItemKind::Literal(_) => false,
             }
         })
     })
 }
 
+fn desugaring_name(des: hir::MatchSource) -> String {
+    match des {
+        hir::MatchSource::ForLoopDesugar => "MatchSource::ForLoopDesugar".to_string(),
+        hir::MatchSource::TryDesugar => "MatchSource::TryDesugar".to_string(),
+        hir::MatchSource::WhileLetDesugar => "MatchSource::WhileLetDesugar".to_string(),
+        hir::MatchSource::Normal => "MatchSource::Normal".to_string(),
+        hir::MatchSource::IfLetDesugar { contains_else_clause } => format!("MatchSource::IfLetDesugar {{ contains_else_clause: {} }}", contains_else_clause),
+    }
+}
+
+fn loop_desugaring_name(des: hir::LoopSource) -> &'static str {
+    match des {
+        hir::LoopSource::ForLoop => "LoopSource::ForLoop",
+        hir::LoopSource::Loop => "LoopSource::Loop",
+        hir::LoopSource::WhileLet => "LoopSource::WhileLet",
+    }
+}
+
 fn print_path(path: &QPath, first: &mut bool) {
     match *path {
         QPath::Resolved(_, ref path) => for segment in &path.segments {
