@@ -24,9 +24,9 @@ use syntax::attr;
 /// visibility inherited from outer item
 /// extern crate dylib source: "/path/to/foo.so"
 /// ```
-declare_lint! {
+declare_clippy_lint! {
     pub DEEP_CODE_INSPECTION,
-    Warn,
+    internal_warn,
     "helper to dump info about code"
 }
 
@@ -250,7 +250,7 @@ fn print_expr(cx: &LateContext, expr: &hir::Expr, indent: usize) {
             println!("{}Yield", ind);
             print_expr(cx, sub, indent + 1);
         },
-        hir::ExprBlock(_) => {
+        hir::ExprBlock(_, _) => {
             println!("{}Block", ind);
         },
         hir::ExprAssign(ref lhs, ref rhs) => {
@@ -274,12 +274,6 @@ fn print_expr(cx: &LateContext, expr: &hir::Expr, indent: usize) {
             println!("{}struct expr:", ind);
             print_expr(cx, e, indent + 1);
         },
-        hir::ExprTupField(ref e, ref idx) => {
-            println!("{}TupField", ind);
-            println!("{}field index: {}", ind, idx.node);
-            println!("{}tuple expr:", ind);
-            print_expr(cx, e, indent + 1);
-        },
         hir::ExprIndex(ref arr, ref idx) => {
             println!("{}Index", ind);
             println!("{}array expr:", ind);
@@ -403,9 +397,9 @@ fn print_item(cx: &LateContext, item: &hir::Item) {
                 println!("trait is not auto");
             }
         },
-        hir::ItemAutoImpl(_, ref _trait_ref) => {
-            println!("auto impl");
-        },
+        hir::ItemTraitAlias(..) => {
+            println!("trait alias");
+        }
         hir::ItemImpl(_, _, _, _, Some(ref _trait_ref), _, _) => {
             println!("trait impl");
         },
